@@ -3,7 +3,7 @@ from biblemate.uba.dialogs import *
 from biblemate.ui.text_area import getTextArea
 from biblemate.ui.info import get_banner
 from biblemate import config, DIALOGS, BIBLEMATE_VERSION, AGENTMAKE_CONFIG, BIBLEMATE_USER_DIR, BIBLEMATEDATA, fix_string, write_user_config, list_dir_content
-from biblemate.uba.api import DEFAULT_MODULES, run_uba_api, run_uba_ai_commentary, run_uba_words, run_uba_discourse, run_uba_translation
+from biblemate.uba.api import DEFAULT_MODULES, run_uba_api
 from pathlib import Path
 import urllib.parse
 import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, warnings, sys, traceback
@@ -12,6 +12,7 @@ from alive_progress import alive_bar
 from fastmcp import Client
 from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
+from agentmake.plugins.uba.lib.BibleParser import BibleVerseParser
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, extractText, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND, DEFAULT_TEXT_EDITOR
 from agentmake.utils.files import searchFolder, isExistingPath
 from agentmake.etextedit import launch_async
@@ -36,7 +37,7 @@ parser = argparse.ArgumentParser(description = f"""BibleMate AI {BIBLEMATE_VERSI
 # global options
 parser.add_argument("default", nargs="*", default=None, help="initial prompt")
 parser.add_argument("-b", "--backend", action="store", dest="backend", help="AI backend; overrides the default backend temporarily.")
-parser.add_argument("-l", "--lite", action="store", dest="lite", choices=["true", "false"], help="Enable / disable lite context. Must be one of: true, false.")
+parser.add_argument("-l", "--light", action="store", dest="light", choices=["true", "false"], help="Enable / disable light context. Must be one of: true, false.")
 parser.add_argument("-m", "--mode", action="store", dest="mode", choices=["agent", "partner", "chat"], help="Specify AI mode. Must be one of: agent, partner, chat.")
 parser.add_argument("-pe", "--promptengineer", action="store", dest="promptengineer", choices=["true", "false"], help="Enable / disable prompt engineering. Must be one of: true, false.")
 parser.add_argument("-s", "--steps", action="store", dest="steps", type=int, help="Specify the maximum number of steps allowed.")
@@ -74,10 +75,10 @@ Please provide me with the final answer to my original request based on the work
 """
 
 # other temporary config changes
-if args.lite == "true":
-    config.lite = True
-elif args.lite == "false":
-    config.lite = False
+if args.light == "true":
+    config.light = True
+elif args.light == "false":
+    config.light = False
 if args.mode == "agent":
     config.agent_mode = True
 elif args.mode == "partner":
@@ -128,12 +129,16 @@ async def initialize_app(client):
     available_tools = [i for i in available_tools if not i in config.disabled_tools]
 
     tool_descriptions = ""
+    tool_descriptions_lite = ""
     if "get_direct_text_response" not in tools:
-        tool_descriptions = """# TOOL DESCRIPTION: `get_direct_text_response`
+        tool_descriptions = tool_descriptions_lite = """# TOOL DESCRIPTION: `get_direct_text_response`
 Get a static text-based response directly from a text-based AI model without using any other tools. This is useful when you want to provide a simple and direct answer to a question or request, without the need for online latest updates or task execution."""
     for tool_name, tool_description in tools.items():
+        tool_description_lite = tool_description.strip().split("\n")[0]
         tool_descriptions += f"""# TOOL DESCRIPTION: `{tool_name}`
 {tool_description}\n\n\n"""
+        tool_descriptions_lite += f"""# TOOL DESCRIPTION: `{tool_name}`
+{tool_description_lite}\n\n\n"""
 
     prompts_raw = await client.list_prompts()
     prompts = {p.name: p.description for p in prompts_raw}
@@ -169,10 +174,10 @@ Get a static text-based response directly from a text-based AI model without usi
     templates = {r.name: (r.description, r.uriTemplate) for r in templates_raw}
     templates = dict(sorted(templates.items()))
     
-    return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates
+    return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, tool_descriptions_lite, prompts, prompts_schema, resources, templates
 
-def display_cancel_message(console):
-    console.print(f"[bold {get_border_style()}]Cancelled![/bold {get_border_style()}]\n")
+def display_cancel_message(console, cancel_message="Cancelled!"):
+    console.print(f"[bold {get_border_style()}]{cancel_message}[/bold {get_border_style()}]\n")
     # display_info(console, "I've stopped processing for you.")
     config.cancelled = True
 
@@ -227,6 +232,13 @@ def get_border_style():
         return config.color_partner_mode
     return "none"
 
+def get_lite_messages(messages, original_request):
+    trimmed_messages = messages[len(DEFAULT_MESSAGES):]
+    lite_messages = [{"role": "user", "content": original_request},{"role": "assistant", "content": "Let's begin."}] if len(trimmed_messages) >= 2 else []
+    if len(trimmed_messages) > 2:
+        lite_messages += trimmed_messages[len(trimmed_messages)-2:]
+    return [{"role": "system", "content": DEFAULT_SYSTEM}]+lite_messages
+
 async def download_data(console, default=""):
     file_ids = {
         "bible.db": "1E6pDKfjUMhmMWjjazrg5ZcpH1RBD8qgW",
@@ -290,7 +302,7 @@ async def main_async():
         console.clear()
         console.print(get_banner(BIBLEMATE_VERSION))
 
-        tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates = await initialize_app(client)
+        tools, tools_schema, master_available_tools, available_tools, tool_descriptions, tool_descriptions_lite, prompts, prompts_schema, resources, templates = await initialize_app(client)
         resource_suggestions_raw = json.loads(run_uba_api(".resources"))
         # check if default modules are valid:
         config_changed = False
@@ -320,6 +332,7 @@ async def main_async():
         resource_suggestions += [f"//lexicon/{i}/" for i in resource_suggestions_raw["lexiconList"]]
         abbr = BibleBooks.abbrev["eng"]
         resource_suggestions += [abbr[str(book)][0] for book in range(1,67)]
+        resource_suggestions += ["."+abbr[str(book)][0]+" " for book in range(1,67)]
 
         write_user_config() # remove the temporary `config.backend`
         
@@ -409,7 +422,7 @@ async def main_async():
             # Original user request
             # note: `python3 -m rich.emoji` for checking emoji
             console.print("Enter your request :smiley: :" if len(messages) == len(DEFAULT_MESSAGES) else "Enter a follow-up request :flexed_biceps: :")
-            input_suggestions = list(config.action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions+config.custom_input_suggestions
+            input_suggestions = list(config.action_list.keys())+[".editprompt", "@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+["//"]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions+config.custom_input_suggestions
             if args.default:
                 user_request = " ".join(args.default).strip()
                 args.default = None # reset to avoid repeated use
@@ -430,12 +443,23 @@ async def main_async():
                         display_info(console, info, title="Error!")
                         config.current_prompt = check_path
                 continue
-            # luanch action menu
+            # process user request
             if not user_request:
                 continue
+            # luanch action menu
             elif user_request == ".":
                 select = await DIALOGS.getValidOptions(options=config.action_list.keys(), descriptions=[i.capitalize() for i in config.action_list.values()], title="Action Menu", text="Select an action:")
                 user_request = select if select else ""
+            # read bible references directly
+            elif user_request.startswith(".") and not ((user_request in config.action_list) or user_request.startswith(".open ") or user_request.startswith(".import ")):
+                user_request = fix_string(user_request[1:])
+                # three cases: 1. verses 2. chapter 3. search bible
+                refs = BibleVerseParser(False).extractAllReferencesReadable(user_request)
+                if refs:
+                    user_request = f"//bible/{refs}" if ":" in user_request else f"//chapter/{refs}"
+                else:
+                    user_request = f"//search/{user_request}"
+            # system commands
             elif user_request.startswith("!"):
                 cmd = user_request[1:].strip()
                 if not cmd:
@@ -619,6 +643,8 @@ async def main_async():
                 except Exception as e: # invalid uri
                     print(f"Error: {e}\n")
                     continue
+            elif user_request.startswith("//"):
+                user_request = user_request[2:]
 
             # system command
             if user_request.startswith(".open") or user_request.startswith(".import") or user_request.startswith(".reload"):
@@ -909,10 +935,10 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
                     write_user_config()
                     info = f"Auto Tool Selection in Chat Mode `{'Enabled' if config.auto_tool_selection else 'Disabled'}`"
                     display_info(console, info, title="configuration")
-                elif user_request == ".lite":
-                    config.lite = not config.lite
+                elif user_request == ".light":
+                    config.light = not config.light
                     write_user_config()
-                    info = f"Lite Context `{'Enabled' if config.lite else 'Disabled'}`"
+                    info = f"Light Context `{'Enabled' if config.light else 'Disabled'}`"
                     display_info(console, info, title="configuration")
                 elif user_request == ".download":
                     await download_data(console)
@@ -992,7 +1018,7 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
             specified_tool = ""
 
             # Tool selection systemm message
-            system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions)
+            system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions_lite if config.light else tool_descriptions)
 
             # auto tool selection in chat mode
             if config.agent_mode is None and config.auto_tool_selection and not user_request.startswith("@"):
@@ -1085,7 +1111,7 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
                             if "```" in user_request:
                                 user_request = re.sub(r"^.*?(```improved_version|```)(.+?)```.*?$", r"\2", user_request, flags=re.DOTALL).strip()
                     except:
-                        improved_prompt_output = agentmake(messages if messages else user_request, follow_up_prompt=user_request if messages else None, system="improve_prompt_2", **AGENTMAKE_CONFIG)
+                        improved_prompt_output = agentmake(messages if messages else user_request, follow_up_prompt=user_request if messages else None, system=get_system_improve_prompt_2(), **AGENTMAKE_CONFIG)
                         if improved_prompt_output:
                             user_request = improved_prompt_output[-1].get("content", "").strip()
                             user_request = re.sub(r"^.*?(```improved_prompt|```)(.+?)```.*?$", r"\2", user_request, flags=re.DOTALL).strip()
@@ -1112,12 +1138,15 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
                     else:
                         user_request = improved_prompt_edit
 
+                # update original request
+                original_request = user_request
+
             # Add user request to messages
             if not user_request == "[CONTINUE]":
                 messages.append({"role": "user", "content": user_request})
 
             async def run_tool(tool, tool_instruction):
-                nonlocal messages
+                nonlocal messages, original_request
                 tool_instruction = fix_string(tool_instruction)
                 messages[-1]["content"] = fix_string(messages[-1]["content"])
                 if tool == "get_direct_text_response":
@@ -1128,7 +1157,7 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
                         tool_properties = tool_schema["parameters"]["properties"]
                         if len(tool_properties) == 1 and "request" in tool_properties: # AgentMake MCP Servers or alike
                             if "items" in tool_properties["request"]: # requires a dictionary instead of a string
-                                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else deepcopy(messages)
+                                request_dict = get_lite_messages(messages, original_request) if config.light else deepcopy(messages)
                                 tool_result = await client.call_tool(tool, {"request": request_dict}, timeout=config.mcp_timeout)
                             else:
                                 tool_result = await client.call_tool(tool, {"request": tool_instruction}, timeout=config.mcp_timeout)
@@ -1226,12 +1255,12 @@ Press `Ctrl+C` once or twice until the loading is cancelled, while you are waiti
 
 Available tools are: {available_tools}.
 
-{tool_descriptions}
+{tool_descriptions_lite if config.light else tool_descriptions}
 
 # My Request
 
 {user_request}"""
-                        master_plan_output = agentmake(messages+[{"role": "user", "content": initial_prompt}], system="create_action_plan", **AGENTMAKE_CONFIG)
+                        master_plan_output = agentmake(messages+[{"role": "user", "content": initial_prompt}], system=get_system_master_plan(), **AGENTMAKE_CONFIG)
                         if master_plan_output:
                             master_plan = master_plan_output[-1].get("content", "").strip()
                     try:
@@ -1269,6 +1298,7 @@ Available tools are: {available_tools}.
             system_make_suggestion = get_system_make_suggestion(master_plan=master_plan)
 
             # Get the first suggestion
+            config.cancelled = False
             conversation_broken = False
             if user_request == "[CONTINUE]":
                 next_suggestion = "CONTINUE"
@@ -1357,10 +1387,26 @@ Available tools are: {available_tools}.
                     else:
                         next_tool_description = tools.get(next_tool, "No description available.")
                         system_tool_instruction = get_system_tool_instruction(next_tool, next_tool_description)
-                        next_step_output = agentmake(next_suggestion, system=system_tool_instruction, **AGENTMAKE_CONFIG)
-                        next_step = next_step_output[-1].get("content", "").strip()
-                        # The following line may give better context, but when a conversation goes long, the agent loses track of the system message.
+                        # No context; missing information for the tool from the conversation
+                        #next_step_output = agentmake(next_suggestion, system=system_tool_instruction, **AGENTMAKE_CONFIG)
+                        #next_step = next_step_output[-1].get("content", "").strip()
+                        # Full context, but when a conversation goes long, the agent loses track of the system message.
                         #next_step = agentmake([{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):], follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                        # Minimum context that includes the original request and the latest exchanges, if any
+                        lite_messages = get_lite_messages(messages, original_request)
+                        next_suggestion += f"""
+
+# Remember:
+* Do NOT provide the answer or perform the task. Provide the instruction ONLY, which the AI assistant will follow or answer.
+* You are here to proved the instruction for the current step ONLY.
+* Do not mention the tool name in your instruction.
+* Do not mention further steps or tools to be used after this instruction.
+* Only provide the instruction for the specified tool `{next_tool}`.
+* Pay attention to the information the tool requires and provide the necessary details in your instruction.
+
+You provide the converted instruction directly, without any additional commentary or explanation."""
+                        next_step_output = agentmake(lite_messages, system=system_tool_instruction, follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)
+                        next_step = next_step_output[-1].get("content", "").strip()
                 try:
                     await thinking(get_next_step, "Crafting the next instruction ...")
                     if not next_step_output:
@@ -1396,6 +1442,11 @@ Available tools are: {available_tools}.
                     display_cancel_message(console)
                     conversation_broken = True
                     break
+                if messages[-1]['content'] == "[NO_CONTENT]":
+                    messages = messages[:-2]  # remove last user and assistant messages
+                    display_cancel_message(console, cancel_message="No content was generated. Stopping the process.")
+                    conversation_broken = True
+                    break
                 console.rule()
                 console.print(Markdown(f"\n## Output [{step}]\n\n{messages[-1]['content']}"))
                 console.print()
